package lv.tsi.oodb;

import lv.tsi.oodb.util.*;
import lv.tsi.oodb.core.*;
import org.odmg.*;

import java.util.*;
import java.io.*;

/**
 * Main class to handle database instances and initialize all global
 * cofigurations.
 *
 * @author  Alexey Vasilyev
 * @version 0.1.0 18 December 2000
 */
public class OODB implements Debug {

    private static final String VERSION    = "OODB v0.1.0";
    private static final String PROPERTIES = "oodb.properties";

    private static OODB current;

    /** Table of currently opened databases **/
    private HashMap    databases;
    /** Properties **/
    private Properties prop;

    /** Log **/
    private Log log;

    /** Transaction manager **/
    private TransactionManager txnmanager;

    /**
     * Main constructor.
     */
    public OODB() throws OODBException {
        current = this;
        init( PROPERTIES );
        log.info( "-------------------------------------------------------" );
        log.info( "[OODB] " + VERSION + " started." );
        databases = new HashMap( 20 );
    }

    /**
     * @param filename  the file with properties
     */
    public void init(String filename) throws OODBException {
        // Initializing properties
        prop = new Properties();
        try {
            InputStream in = new FileInputStream( filename );
            prop.load( in );
            in.close();
        } catch (IOException e) {
            System.out.println("OODB::init() " + e);
            throw new OODBException( "File '" + PROPERTIES + "' not found." );
        }

        // Initializing log system
        log = new Log( prop.getProperty("log.filename"), 
                       Log.getLogLevel(prop.getProperty("log.level")) );

        txnmanager = new TransactionManager();

        // Init shutdown hook. For JDK 1.3 Only!
        /*Runtime.getRuntime().addShutdownHook( new Thread() {
            public void run() { OODB.this.shutdown(); }
        });*/
    }

    /**
     * Shutdown all opened connections.
     */
    public void shutdown() {
        log.close();
        log.info( "[OODB] OODB shutdowned." );
        log.info( "-------------------------------------------------------" );
    }

    /**
     * @param filename  the file with properties
     */
    public void bind(String name, Database db) {
        databases.put( name, db );
    }

    /**
     * @param filename  the file with properties
     */
    public void unbind(String name) {
        databases.remove( name );
    }

    public Log getLog() {
        return log;
    }

    public TransactionManager getTransactionManager() {
        return txnmanager;
    }

    /**
     * Debug implementation.
     */
    public void dump() {
        log.debug( "[OODB] Databases#: " + databases.size() );
    }

///////////////////////////
// Static methods
//
    /**
     * Create instance of the OODB class if it doesn't exist.
     * Is called from Database open(..) method.
     */
    public static void init() throws OODBException {
        if ( current == null )
            new OODB();
    }

    public static OODB getCurrent() {
        return current;
    }

    /**
     * Check if the given object is of type MetaObject.
     *
     * @param object  the object to check.
     * @throws        OODBRuntimeException if the object is not of type
     *                MetaObject.
     */
    public static void isMetaObject(Object object) {
        // Check type of object
        if ( !(object instanceof MetaObject) )
            throw new OODBRuntimeException("Object's class must be of type MetaObject.");
    }

    public static void main(String[] s) {
        try {
            new OODB();
        } catch (OODBException e) {
        }
    }

}
