package lv.tsi.oodb.core;

import lv.tsi.oodb.odmg.*;
import org.odmg.*;
import java.util.*;

/**
 * A class extent is a collection of all objects in the database of a given
 * class type. At this time, neither the Java language nor the ODMG Java
 * binding defines a class extent. However, this OODB provides such a class,
 * the Extent type.
 *
 * Each getting request for getting object via Extent should call clone()
 * of meta object which increments it's version.
 *
 * @author  Alexey Vasilyev
 * @version 0.0.2 16 December 2000
 */
public class Extent implements Iterator {
    
    private Enumeration enum;

    /**
     * Create an instance of this class with all persistent objects
     * 
     *
     * @param className   class name which instances should be returned.
     */
    public Extent( String className ) {
        this( lv.tsi.oodb.odmg.Transaction.getCurrent(), className );
    }

    /**
     * @param tr          transaction to open.
     * @param className   class name which instances should be returned.
     */
    public Extent( lv.tsi.oodb.odmg.Transaction tr, String className ) {
        if (tr.isOpen()) {
            Vector v = tr.getDatabase().getObjectManager().getPersistentObjects( className );
            //  ----------------------------------------------------
            // | Register all returned objects as transient.        |
            //  ----------------------------------------------------
            enum = v.elements();
            while (enum.hasMoreElements()) {
                tr.registerTransientObject( (MetaObject)enum.nextElement() );
            }
            enum = v.elements();
        }
    }

    /**
     * Returns true if the extent contains further elements.
     */
    public boolean hasNext() {
        return enum.hasMoreElements();
    }

    /**
     * Returns the next element of the extent.
     * Calls to this method enumerate successive elements.
     */
    public Object next() {
        return enum.nextElement();
    }

    /**
     * No insert or remove methods are involved because its content is automatically
     * maintained by the ODBMS. 
     */
    public void remove() {}

}
