package lv.tsi.oodb.core;

/**
 * Object identificator.
 * Each object ID consists of two parts:
 *   1. The creation time in milliseconds.
 *   2. The number of object if the object of this creation time allready exits.
 *
 * There is a problem in Java. The smallest time interval is 1 ms. But the
 * object creation time is less than 1 ms. Since why we implemented salt here.
 * If the previous creation time is equaled to current then we increment salt.
 * For example, the string output to 5 sequentially created objects can be:
 *   [973984752200,0]
 *   [973984752200,1]
 *   [973984752200,2]
 *   [973984752310,0]
 *   [973984752310,1]
 *
 * @author  Alexey Vasilyev
 * @version 0.0.4 19 November 2000
 */
public final class OID implements java.io.Serializable {

    private long time;
    private int  salt;

    /**
     * Create OID instance with given values.
     *
     * @param time      object creation time.
     * @param salt      object salt.
     */
    protected OID(long time, int salt) {
        this.time = time;
        this.salt = salt;
    }

    /**
     * Get the OID creation time.
     */
    public final long getTime() { return time; }

    /**
     * Get the OID salt.
     */
    public final int  getSalt() { return salt; }

    public String toString() {
        return "[" + time + "," + salt + "]";
    }

    /**
     * Hash code algorithm is based on java.lang.Long class hash algorithm.
     */
    public int hashCode() {
        return (int)(time ^ (time >> 32)) ^ salt;
        //return (int)(time ^ (time >> 32)) ^ salt ^ (salt << 16);
    }

}
