package lv.tsi.oodb.core;

import lv.tsi.oodb.OODB;
import lv.tsi.oodb.odmg.*;
import lv.tsi.oodb.util.Log;
import org.odmg.TransactionInProgressException;

import java.util.*;

/**
 * Object manager core component implementation.
 * Handles registry of allocated objects.
 *
 * @author  Alexey Vasilyev
 * @version 0.0.2 5 November 2000
 */
public class TransactionManager implements Debug {

    /** Handles transaction and thread tables **/
    private TransactionHashMap map = new TransactionHashMap();

    public TransactionManager() {
        //current = this;
    }

    public void init(String name) throws Exception {
    }

    /**
     * Register transaction in the transaction table.
     *
     * @param tr  opened transaction to bind
     */
    public synchronized void register(Transaction tr) throws TransactionInProgressException {
        map.put( Thread.currentThread(), tr );
    }

    /**
     * Remove transaction from the transaction table.
     *
     * @param th  thread where transaction exists
     */
    public synchronized void unregister( Transaction tr ) throws TransactionInProgressException {
        map.remove( tr );
    }

    /**
     * Remove thread from the transaction table.
     *
     * @param th  thread where transaction exists
     */
    public synchronized void remove( Thread th ) throws TransactionInProgressException {
        map.remove( th );
    }

    public String toString() {
        return map.toString();
    }

    /**
     * Printing this istance variables.
     */
    public void dump() {
        map.dump();
    }

///////////////////////////
// Static methods
//
    public static TransactionManager getCurrent() {
        return OODB.getCurrent().getTransactionManager();
    }

    public static Transaction getTransaction( Thread th ) {
        return getCurrent().map.getTransaction( th );
    }

}
