package lv.tsi.oodb.core;

import lv.tsi.oodb.odmg.*;
import lv.tsi.oodb.util.Log;
import org.odmg.ObjectNameNotUniqueException;

import java.util.*;

/**
 * Object manager core component implementation.
 * Is used by object manager.
 *
 * @author  Alexey Vasilyev
 * @version 0.0.4 19 November 2000
 */
public class TransientNamedHashMap {//implements Debug {

    private final static int HASH_INIT = 20;

    /** The table of currently binded objects.
        Key=OID(MetaObject), Value=String **/
    private HashMap objects  = new HashMap( HASH_INIT );
    /** Set of names binded. Key=String, Value=MetaObject **/
    private HashMap names    = new HashMap( HASH_INIT );

    public TransientNamedHashMap() {
    }

    /**
     * Register transaction in the transaction table.
     * Is called from (Transaction)txn.begin() and (Transaction)txn.join()
     *
     * @param th  thread where transaction exists
     * @param tr  opened transaction to bind
     */
    public synchronized void put(NamedWrapper object, String name) throws ObjectNameNotUniqueException {
        if (names.get(name) != null)
            throw new ObjectNameNotUniqueException( "Error." );
        names.put( name, object );
        objects.put( object.getOID(), name );
    }

    /**
     * Remove thread from the transaction and thread tables.
     * Is called from (Transaction)txn.leave()
     *
     * @param th  thread to remove
     */
    public synchronized NamedWrapper remove(MetaObject object) {
        String name = (String)objects.remove( object.getOID() );
        return (NamedWrapper)names.remove( name );
    }

    /**
     * Remove thread from the transaction and thread tables.
     * Is called from (Transaction)txn.leave()
     *
     * @param th  thread to remove
     */
    public synchronized void remove(String name) {
        NamedWrapper obj = (NamedWrapper)names.remove( name );
        names.remove( obj.getOID() );
    }

    /**
     * Debug implementation.
     */
    //public void dump() {
    //    Log.getCurrent().debug( "[OBJMNGR]\tNames: " + threads.size() + "\tTransactions#: " + transactions.size() );
    //}

    public String get(MetaObject object) {
        return (String)objects.get(object);
    }

    public NamedWrapper get(String name) {
        return (NamedWrapper)names.get(name);
    }

    public int size() {
        return names.size();
    }

}
