package lv.tsi.oodb.db.orders;

import lv.tsi.oodb.odmg.*;
import lv.tsi.oodb.core.MetaObject;
import java.util.*;

/**
 * Item Class. Represents item business object of our scheme.
 * Each SubOrder consists at least of one ITEM.
 * ITEM hold some identical(!!!) products.
 * For example 52 Hansol Monitors or 14 Panasonic KX-14 Phones
 * @author Alex Gulaev
 * @versio 0.0.1 11 November 2000
 */
public class Item extends MetaObject {
    
  
    private int quantity;
    private String parameter;
    private float price;
    private float discount;
    private String description;
    private Date updDate;
    private String sku;
    private float srcPrice;
    

    // ****** Relations ******

    private SubOrder suborder;
    private Product product;
    private User updater;
    private Status status;
    private SetOfObject itemDetails = new SetOfObject();

    //************************
    public Item(){}
    
    public void init( int quantity, String parameter, float price,
                      float discount, String sku, float srcPrice) {
        
       init( quantity, parameter, price, discount, "", new Date(), sku, srcPrice );
    }

    public void init( int quantity, String parameter, float price,
                      float discount, Date updDate, String sku, float srcPrice) {
        
       init( quantity, parameter, price, discount, "", updDate, sku, srcPrice );
    }
    

    
    public void init( int quantity, String parameter, float price,
                      float discount, String description, Date updDate,
                      String sku, float srcPrice) {
        
       this.quantity      = quantity;
       this.parameter     = parameter;
       this.price         = price;
       this.discount      = discount;
       this.description   = description;
       this.updDate       = updDate;
       this.sku           = sku;
       this.srcPrice      = srcPrice;
      
    }
        
    
     public void setQuantity(int quantity){    
       this.quantity      = quantity;
     }  
       
     public void setParameter(String parameter){ 
       this.parameter     = parameter;
     }
       
     public void setPrice(float price){  
       this.price         = price;
     }
       
     public void setDiscount(float discount){  
       this.discount      = discount;
     }  
       
     public void setDescription(String description){
       this.description   = description;
     }  
       
     public void setUpdDate(Date updDate){
       this.updDate       = updDate;
     }  

     public void setSku(String sku){
       this.sku           = sku;
     }  

     public void setSrcPrice(float srcPrice){
       this.srcPrice      = srcPrice;
     }  
    
    

     
     public int getQuantity(){    
       return quantity;
     }  
       
     public String getParameter(){ 
       return parameter;
     }
       
     public float getPrice(){  
       return price;
     }
       
     public float getDiscount(){  
       return discount;
     }  
       
     public String getDescription(){
       return description;
     }  
       
     public Date getUpdDate(){
       return updDate;
     }  

     public String getSku(){
       return sku;
     }  

     public float getSrcPrice(){
       return srcPrice;
     }
     
     
       
   
    public void addItemDetail ( ItemDetail itd ) {
        itemDetails.add( itd );
    }

    public void linkToStatus( Status stat ) {
        status = stat;
    }

    public void linkToUpdater( User usr ) {
        updater = usr;
    }
   
    public void linkToProduct( Product prod ) {
        product = prod;
    }
       
    public void linkToSubOrder( SubOrder sb ) {
        suborder = sb;
    }
   


    
    public SetOfObject getItemDetails () {
        return itemDetails;
    }

    public Status getStatus() {
        return status;
    }

    public User getUpdater() {
        return updater;
    }
   
    public Product getProduct() {
        return product;
    }
       
    public SubOrder getSubOrder() {
        return suborder;
    }

    
    
    
    
    
    
    public String toString() {
        return "Item id: " + this.getOID().toString();
    }

}
