package lv.tsi.oodb.db.orders;

import lv.tsi.oodb.odmg.*;
import lv.tsi.oodb.core.MetaObject;
import java.util.*;

/**
 * SubOrder Class. Represents sub order business object of our scheme
 *
 * @author Alex Gulaev
 * @version 0.0.4 02 December 2000
 */
public class SubOrder extends Order {
      
    // Attributes of Class SubOrder (see other in Order class)
    private String pon;
    private float dlvPrice;
    private String shipperName;
    private String paymentObjID;
    private String paymentClassID;

    //***** Relations *******
    private Order order;
    private SetOfObject items = new SetOfObject();
    private SetOfObject users = new SetOfObject();
    private Status status;
    private LinkDoc linkDoc;
    private Delivery delivery;
    //************************



    public SubOrder(){}

    
    //***********************************************************
    //** Methods, which are the analog of SubOrder constructor **
    //***********************************************************
     
    public void init( String number, String type, Date createDate, Date updDate,
                      String pon, float dlvPrice, String shipperName,
                      String paymentObjID, String paymentClassID ) {
       
       this.number         = number;
       this.type           = type;
       this.createDate     = createDate;
       this.updDate        = updDate; 
       this.pon            = pon;
       this.dlvPrice       = dlvPrice;
       this.shipperName    = shipperName;
       this.paymentObjID   = paymentObjID;
       this.paymentClassID = paymentClassID;
   
    }

    // use default date = current and pon="";
    public void init( String number, String type, String pon,
                      float dlvPrice, String shipperName, String paymentObjID,
                                      String paymentClassID ) {

       init( number, type, new Date(), new Date(), pon, 
             dlvPrice, shipperName, paymentObjID, paymentClassID );
   
    }
    
    
    //*************************************************************
    //** Methods, which set each available attribute of SubOrder **
    //*************************************************************
         
     public void setPon(String pon){  
       this.pon            = pon;
     }
    
     public void setDlvPrice(float dlvPrice){  
       this.dlvPrice       = dlvPrice;
     }

     public void setPaymentObjID(String paymentObjID){  
       this.paymentObjID   = paymentObjID;
     } 

     public void setPaymentClassID(String paymentClassID){  
       this.paymentClassID = paymentClassID;
     }
    
    
    //*************************************************************
    //** Methods, which get values of SubOrder attributes        **
    //*************************************************************
        
     public String getPon(){
       return pon;
     }

     /**
      * Processed field.
      */
     public float getTotalSum(){
       float sum = 0;
       Iterator i = items.iterator();
       while (i.hasNext()) {
          Item it = (Item)i.next();
          sum += it.getQuantity() * it.getPrice();
       }
       return sum;
     }

     public float getTotalDiscount(){

       float sum = 0;
       Iterator i = items.iterator();
       while (i.hasNext()) {
          Item it = (Item)i.next();
          sum += it.getQuantity() * it.getDiscount();
       }
       return sum; 
     
     }
     //------------------------------------------------
     
     
     public float getDlvPrice(){
       return dlvPrice;
     }

     public String getPaymentObjID(){
       return paymentObjID;
     }

     public String getPaymentClassID(){
       return paymentClassID;
     }
     
    
    
    
    //**********************************************************
    //** Methods, which assign appropriate object to SubOrder **
    //**********************************************************
    
    // Special Document associated with this SubOrder
    public void linkToLinkDoc( LinkDoc ld ) {
        linkDoc = ld;
    }

    // Add items to SubOrder (One item represents any quantity of one product)
    public void addItem( Item itm ) {
        items.add( itm );
    }
    
    // Relation to Parent Order of this SubOrder
    public void linkToOrder( Order ord ) {
        order = ord;
    }

    // Relation to Delivery of this SubOrder
    public void linkToDelivery( Delivery dlv ) {
        delivery = dlv;
    }

    //**********************************************************
    //** Methods, which get appropriate object of SubOrder    **
    //**********************************************************
    
    public LinkDoc getLinkDoc() {
        return linkDoc;
    }

    public SetOfObject getItems() {
        return items;
    }
    
    public Order getOrder() {
        return order;
    }

    public Delivery getDelivery() {
        return delivery;
    }

    
    // Return OID of this instance
    public String toString() {
        return "SubOrder id: " + this.getOID().toString() +", Number: " + this.number;
    }

}
