package lv.tsi.oodb.debug;

import lv.tsi.oodb.odmg.*;
import lv.tsi.oodb.core.OODBRuntimeException;

import org.odmg.ODMGException;
import org.odmg.ObjectNameNotUniqueException;
import org.odmg.ObjectNameNotFoundException;

/**
 * ODMG classes implementation test.
 *
 * @author  Alexey Vasilyev
 * @version 0.0.1 1 November 2000
 */
public class OODBTest {

    /**
     * Binding named object to the database.
     */
    private static void bind(Database db, String name) throws ODMGException {
        // everything must be done inside a transaction
        Transaction txn = new Transaction( db );
        txn.begin();

        try {
            MyClass myObject = new MyClass();
            db.bind( myObject, name );
        } catch (ObjectNameNotUniqueException exc) {
            // in case something went wrong, we abort the transaction
            // this is especially important to release all locks
            // then we "rethrow" the exception
            txn.abort();
            throw exc;
        /*} catch (OODBRuntimeException exc) {
            // Runtime exceptions need not to be declared and can be thrown anytime.
            // Two important examples are:
            //      * connection to server lost
            //      * lock not granted (object locked by another client)
            txn.abort();
            throw exc;*/
        }

        txn.commit();
    }

    /**
     * Obtaining named object from the database.
     */
    private static String lookup(Database db, String name) throws ODMGException {
        String ret = null;

        // everything must be done inside a transaction
        Transaction txn = new Transaction( db );
        txn.begin();

        try {
            MyClass myObject = (MyClass)db.lookup(name);
            if (ret != null)
                ret = myObject.toString();
        } catch (ObjectNameNotFoundException exc) {
            // in case something went wrong, we abort the transaction
            // this is especially important to release all locks
            // then we "rethrow" the exception
            txn.abort();
            throw exc;
        /*} catch (OODBRuntimeException exc) {
            // Runtime exceptions need not to be declared and can be thrown anytime.
            // Two important examples are:
            //      * connection to server lost
            //      * lock not granted (object locked by another client)
            txn.abort();
            throw exc;*/
        }

        txn.commit();
        return ret;
    }

////////////////////////////////////
//
    private static String OBJECT_NAME = "Test";

	public static void main( String[] str ) {
        Database db = new Database();
        //  ------------------------------------------------
        // | Opening the database
        //  ------------------------------------------------
        try {
    	    System.out.print( "Opening database..." );
            db.open( "database", Database.OPEN_READ_WRITE );
    	    System.out.println( "ok." );
        } catch (ODMGException e) {
            System.out.println( "error.\nExited." );
    	    System.out.print( e.getMessage() );
    	    System.exit(1);
        }
        //  ------------------------------------------------
        // | Binding object to the database
        //  ------------------------------------------------
    	try {
    	    System.out.print( "Binding object to the database..." );
	        bind( db, OBJECT_NAME );
    	    System.out.println( "ok." );
	    } catch (ODMGException e) {
    	    System.out.println( "error." );
    	    System.out.print( e.getMessage() );
     	}
        //  ------------------------------------------------
        // | Lookuping object in the database
        //  ------------------------------------------------
    	try {
    	    System.out.print( "Lookup object in the database..." );
	        String obj = lookup( db, OBJECT_NAME );
    	    System.out.println( "ok." );
    	    System.out.println( "Object: " + obj );
	    } catch (ODMGException e) {
    	    System.out.print( "error." );
    	    System.out.print( e.getMessage() );
     	}
        //  ------------------------------------------------
        // | Closing the database
        //  ------------------------------------------------
        try {
    	    System.out.print( "Closing database..." );
		    db.close();
    	    System.out.println( "ok." );
        } catch (ODMGException e) {
    	    System.out.println( "error.\nExited." );
    	    System.out.print( e.getMessage() );
        }

        //  ------------------------------------------------
        // | Testing class
        //  ------------------------------------------------
        try {
    	    System.out.print( "Test..." );
    	    MyClass my = new MyClass();
    	    java.lang.reflect.Field f = my.getClass().getDeclaredField("i");
    	    System.out.println( "field=" + f.get(my) );    	
        } catch (Exception e) {
    	    System.out.print( e.getMessage() );
        }
	}

}