package lv.tsi.oodb.odmg;

import org.odmg.*;
import java.util.Enumeration;

/**
 * Multiset (bag).
 *
 * This class represents relationship 1:M
 * There can be not-unique values and the bag is not ordered.
 * Equivalent to ODMG 3.0
 *   BAG <Type>
 *
 * @author Alexey Vasilyev
 * @version 0.0.2 12 November 2000
 */
public class BagOfObject extends java.util.Vector implements org.odmg.DBag {

    /**
     * UNION.
     * For example, we have two bags A(current) and B(otherBag).
     * A = (1,2,3,4). B = (2,3,4,5).
     *    A U B = (1,2,2,3,3,4,4,5)
     *
     * @return new DBag object which is the union of the two sets: the current and the given
     */
    public DBag union( DBag otherBag ) {
        DBag bag = (DBag)this.clone();
        bag.addAll( otherBag );
        return bag;
    }

    /**
     * INTERSECTION.
     * For example, we have two bags A(current) and B(otherBag).
     * A = (1,2,3,4). B = (2,3,4,5).
     *    A ^ B = (2,3,4)
     *
     * @return new DBag object which is the intersection of the two sets: the current and the given
     */
    public DBag intersection( DBag otherBag ) {
        DBag bag = (DBag)this.clone();
        bag.retainAll( otherBag );
        return bag;
    }

    /**
     * DIFFERENCE.
     * For example, we have two bags A(current) and B(otherBag).
     * A = (1,2,2,3,4). B = (2,2,3,4,5).
     *    A - B = (1)
     *
     * @return new DBag object which is the difference of the two sets: the current and the given
     */
    public DBag difference( DBag otherBag ) {
        DBag bag = (DBag)this.clone();
        bag.removeAll( otherBag );
        return bag;
    }

    /**
     * For example, we have a bag A(current).
     * A = (1,1,2,2,2,3,4).
     *    A occurences(2) = 3
     *
     * @return returns the number of occurrences of the object obj in the DBag collection.
     */
    public int occurrences( Object obj ) {
        int i = 0;
        Enumeration e = elements();
        while (e.hasMoreElements())
            if (e.nextElement().equals(obj)) i++;
        return i;
    }

// DCollection

    /**
     * @param predicate   OQL predicate. For example, "student=5".
     */
    public Object selectElement(String predicate) throws QueryInvalidException {
        // Not implemented yet
        throw new NotImplementedException ("OQL not supported");
    }

    public java.util.Iterator select(String predicate) throws QueryInvalidException {
        // Not implemented yet
        throw new NotImplementedException ("OQL not supported");
    }

    public DCollection query(String predicate) throws QueryInvalidException {
        // Not implemented yet
        throw new NotImplementedException ("OQL not supported");
    }

    public boolean existsElement(String predicate) throws QueryInvalidException {
        // Not implemented yet
        throw new NotImplementedException ("OQL not supported");
    }

}